==> onegov.gazette-1.13.1-py3-none-any.whl: onegov/gazette/views/notice_states.py <==
from mailthon.enclosure import Attachment
from morepath import redirect
from onegov.core.security import Personal
from onegov.core.security import Private
from onegov.core.templates import render_template
from onegov.gazette import _
from onegov.gazette import GazetteApp
from onegov.gazette.forms import EmptyForm
from onegov.gazette.forms import RejectForm
from onegov.gazette.layout import Layout
from onegov.gazette.layout import MailLayout
from onegov.gazette.models import GazetteNotice
from onegov.gazette.views import get_user_and_group
from webob.exc import HTTPForbidden


def send_publish_mail(request, notice):
    """ Sends a mail to the publisher with the contents of the notice.

    We use named temporary files because the files stored by depot does not
    have a nice filename (which is automatically used by mailton) and to allow
    temporary files from the memory without a real file on the disk.

    """

    def construct_subject(notice):
        issues = notice.issue_objects
        number = issues[0].number if issues else ''

        organization = notice.organization_object
        parent = organization.parent if organization else None
        parent_id = (parent.external_name or '') if parent else ''

        return "{} {} {} {}".format(number, parent_id, notice.title, notice.id)

    reply_to = (
        request.app.principal.publish_from or
        request.app.mail_sender
    )

    subject = construct_subject(notice)
    content = render_template(
        'mail_publish.pt',
        request,
        {
            'title': subject,
            'model': notice,
            'layout': MailLayout(notice, request)
        }
    )

    attachments = []
    for file in notice.files:
        attachment = Attachment(file.reference.file._file_path)
        content_disposition = 'attachment; filename="{}"'.format(file.name)
        attachment.headers['Content-Disposition'] = content_disposition
        attachments.append(attachment)

    request.app.send_email(
        subject=subject,
        receivers=(request.app.principal.publish_to, ),
        reply_to=reply_to,
        content=content,
        attachments=attachments
    )


@GazetteApp.form(
    model=GazetteNotice,
    name='submit',
    template='form.pt',
    permission=Personal,
    form=EmptyForm
)
def submit_notice(self, request, form):
    """ Submit a notice.

    This view is used by the editors to submit their drafts for the publishers
    to review.

    Only drafted notices may be submitted. Editors may only submit their own
    notices (publishers may submit any notice).

    If a notice has invalid/past issues or an invalid/inactive
    category/organization, the user is redirected to the edit view.

    """

    layout = Layout(self, request)
    is_private = request.is_private(self)

    if not is_private:
        user_ids, group_ids = get_user_and_group(request)
        if not ((self.group_id in group_ids) or (self.user_id in user_ids)):
            raise HTTPForbidden()

    if self.state != 'drafted' and self.state != 'rejected':
        return {
            'layout': layout,
            'title': self.title,
            'subtitle': _("Submit Official Note"),
            'callout': _(
                "Only drafted or rejected official notices may be submitted."
            ),
            'show_form': False
        }

    if (
        self.expired_issues or
        (self.overdue_issues and not is_private) or
        self.invalid_category or
        self.invalid_organization
    ):
        return redirect(request.link(self, name='edit'))

    if form.submitted(request):
        self.submit(request)
        request.message(_("Official notice submitted."), 'success')
        return redirect(layout.dashboard_or_notices_link)

    return {
        'message': _(
            'Do you really want to submit "${item}"?',
            mapping={'item': self.title}
        ),
        'layout': layout,
        'form': form,
        'title': self.title,
        'subtitle': _("Submit Official Note"),
        'button_text': _("Submit Official Note"),
        'cancel': request.link(self)
    }


@GazetteApp.form(
    model=GazetteNotice,
    name='accept',
    template='form.pt',
    permission=Private,
    form=EmptyForm
)
def accept_notice(self, request, form):
    """ Accept a notice.

    This view is used by the publishers to accept a submitted notice.

    Only submitted notices may be accepted.

    """

    layout = Layout(self, request)

    if self.state != 'submitted':
        return {
            'layout': layout,
            'title': self.title,
            'subtitle': _("Accept Official Note"),
            'callout': _("Only submitted official notices may be accepted."),
            'show_form': False
        }

    if (
        self.expired_issues or
        self.invalid_category or
        self.invalid_organization
    ):
        return redirect(request.link(self, name='edit'))

    if form.submitted(request):
        self.accept(request)
        request.message(_("Official notice accepted."), 'success')
        if request.app.principal.publish_to:
            send_publish_mail(request, self)
            self.add_change(request, _("mail sent"))
        return redirect(layout.dashboard_or_notices_link)

    return {
        'message': _(
            'Do you really want to accept "${item}"?',
            mapping={'item': self.title}
        ),
        'layout': layout,
        'form': form,
        'title': self.title,
        'subtitle': _("Accept Official Note"),
        'button_text': _("Accept Official Note"),
        'cancel': request.link(self)
    }


@GazetteApp.form(
    model=GazetteNotice,
    name='reject',
    template='form.pt',
    permission=Private,
    form=RejectForm
)
def reject_notice(self, request, form):
    """ Reject a notice.

    This view is used by the publishers to reject a submitted notice.

    Only submitted notices may be rejected.

    """

    layout = Layout(self, request)

    if self.state != 'submitted':
        return {
            'layout': layout,
            'title': self.title,
            'subtitle': _("Reject Official Note"),
            'callout': _("Only submitted official notices may be rejected."),
            'show_form': False
        }

    if form.submitted(request):
        self.reject(request, form.comment.data)
        request.message(_("Official notice rejected."), 'success')
        if self.user:
            request.app.send_email(
                subject=request.translate(
                    _(
                        "Official Notice Rejected ${id}",
                        mapping={'id': self.id}
                    )
                ),
                receivers=(self.user.username, ),
                reply_to=request.app.mail_sender,
                content=render_template(
                    'mail_notice_rejected.pt',
                    request,
                    {
                        'title': request.translate(_(
                            "Official Notice Rejected ${id}",
                            mapping={'id': self.id}
                        )),
                        'model': self,
                        'comment': form.comment.data,
                        'layout': MailLayout(self, request),
                        'url': request.link(self)
                    }
                )
            )
        return redirect(layout.dashboard_or_notices_link)

    return {
        'message': _(
            'Do you really want to reject "${item}"?',
            mapping={'item': self.title}
        ),
        'layout': layout,
        'form': form,
        'title': self.title,
        'subtitle': _("Reject Official Note"),
        'button_text': _("Reject Official Note"),
        'button_class': 'alert',
        'cancel': request.link(self)
    }


@GazetteApp.form(
    model=GazetteNotice,
    name='publish',
    template='form.pt',
    permission=Private,
    form=EmptyForm
)
def publish_notice(self, request, form):
    """ Publish a notice.

    This view is used by the publishers to publish an accepted notice.

    Only accepted notices may be published.

    """

    layout = Layout(self, request)

    if self.state != 'accepted':
        return {
            'layout': layout,
            'title': self.title,
            'subtitle': _("Publish Official Note"),
            'callout': _("Only accepted official notices may be published."),
            'show_form': False
        }

    if form.submitted(request):
        self.publish(request)
        request.message(_("Official notice published."), 'success')
        return redirect(layout.dashboard_or_notices_link)

    return {
        'message': _(
            (
                'Do you really want to publish "${item}"? This will assign '
                'the publication numbers for the following issues: ${issues}.'
            ),
            mapping={
                'item': self.title,
                'issues': ', '.join(self.issues.keys())
            }
        ),
        'layout': layout,
        'form': form,
        'title': self.title,
        'subtitle': _("Publish Official Note"),
        'button_text': _("Publish Official Note"),
        'cancel': request.link(self)
    }

==> onegov.gazette-1.13.7-py3-none-any.whl: onegov/gazette/views/notice_states.py <==
from mailthon.enclosure import Attachment
from morepath import redirect
from onegov.core.security import Personal
from onegov.core.security import Private
from onegov.core.templates import render_template
from onegov.gazette import _
from onegov.gazette import GazetteApp
from onegov.gazette.forms import EmptyForm
from onegov.gazette.forms import RejectForm
from onegov.gazette.layout import Layout
from onegov.gazette.layout import MailLayout
from onegov.gazette.models import GazetteNotice
from onegov.gazette.views import get_user_and_group
from webob.exc import HTTPForbidden


def send_publish_mail(request, notice):
    """ Sends a mail to the publisher with the contents of the notice.

    We use named temporary files because the files stored by depot does not
    have a nice filename (which is automatically used by mailton) and to allow
    temporary files from the memory without a real file on the disk.

    """

    def construct_subject(notice, request):
        issues = notice.issue_objects
        number = issues[0].number if issues else ''

        organization = notice.organization_object
        parent = organization.parent if organization else None
        parent_id = (parent.external_name or '') if parent else ''
        prefix = ''
        if notice.print_only:
            prefix = "{} - ".format(request.translate(_("Print only")))

        return "{}{} {} {} {}".format(
            prefix, number, parent_id, notice.title, notice.id
        )

    reply_to = (
        request.app.principal.publish_from or
        request.app.mail_sender
    )

    subject = construct_subject(notice, request)
    content = render_template(
        'mail_publish.pt',
        request,
        {
            'title': subject,
            'model': notice,
            'layout': MailLayout(notice, request)
        }
    )

    attachments = []
    for file in notice.files:
        attachment = Attachment(file.reference.file._file_path)
        content_disposition = 'attachment; filename="{}"'.format(file.name)
        attachment.headers['Content-Disposition'] = content_disposition
        attachments.append(attachment)

    request.app.send_email(
        subject=subject,
        receivers=(request.app.principal.publish_to, ),
        reply_to=reply_to,
        content=content,
        attachments=attachments
    )


@GazetteApp.form(
    model=GazetteNotice,
    name='submit',
    template='form.pt',
    permission=Personal,
    form=EmptyForm
)
def submit_notice(self, request, form):
    """ Submit a notice.

    This view is used by the editors to submit their drafts for the publishers
    to review.

    Only drafted notices may be submitted. Editors may only submit their own
    notices (publishers may submit any notice).

    If a notice has invalid/past issues or an invalid/inactive
    category/organization, the user is redirected to the edit view.

    """

    layout = Layout(self, request)
    is_private = request.is_private(self)

    if not is_private:
        user_ids, group_ids = get_user_and_group(request)
        if not ((self.group_id in group_ids) or (self.user_id in user_ids)):
            raise HTTPForbidden()

    if self.state != 'drafted' and self.state != 'rejected':
        return {
            'layout': layout,
            'title': self.title,
            'subtitle': _("Submit Official Note"),
            'callout': _(
                "Only drafted or rejected official notices may be submitted."
            ),
            'show_form': False
        }

    if (
        self.expired_issues or
        (self.overdue_issues and not is_private) or
        self.invalid_category or
        self.invalid_organization
    ):
        return redirect(request.link(self, name='edit'))

    if form.submitted(request):
        self.submit(request)
        request.message(_("Official notice submitted."), 'success')
        return redirect(layout.dashboard_or_notices_link)

    return {
        'message': _(
            'Do you really want to submit "${item}"?',
            mapping={'item': self.title}
        ),
        'layout': layout,
        'form': form,
        'title': self.title,
        'subtitle': _("Submit Official Note"),
        'button_text': _("Submit Official Note"),
        'cancel': request.link(self)
    }


@GazetteApp.form(
    model=GazetteNotice,
    name='accept',
    template='form.pt',
    permission=Private,
    form=EmptyForm
)
def accept_notice(self, request, form):
    """ Accept a notice.

    This view is used by the publishers to accept a submitted notice.

    Only submitted notices may be accepted.

    """

    layout = Layout(self, request)

    if self.state != 'submitted':
        return {
            'layout': layout,
            'title': self.title,
            'subtitle': _("Accept Official Note"),
            'callout': _("Only submitted official notices may be accepted."),
            'show_form': False
        }

    if (
        self.expired_issues or
        self.invalid_category or
        self.invalid_organization
    ):
        return redirect(request.link(self, name='edit'))

    if form.submitted(request):
        self.accept(request)
        request.message(_("Official notice accepted."), 'success')
        if request.app.principal.publish_to:
            send_publish_mail(request, self)
            self.add_change(request, _("mail sent"))
        return redirect(layout.dashboard_or_notices_link)

    return {
        'message': _(
            'Do you really want to accept "${item}"?',
            mapping={'item': self.title}
        ),
        'layout': layout,
        'form': form,
        'title': self.title,
        'subtitle': _("Accept Official Note"),
        'button_text': _("Accept Official Note"),
        'cancel': request.link(self)
    }


@GazetteApp.form(
    model=GazetteNotice,
    name='reject',
    template='form.pt',
    permission=Private,
    form=RejectForm
)
def reject_notice(self, request, form):
    """ Reject a notice.

    This view is used by the publishers to reject a submitted notice.

    Only submitted notices may be rejected.

    """

    layout = Layout(self, request)

    if self.state != 'submitted':
        return {
            'layout': layout,
            'title': self.title,
            'subtitle': _("Reject Official Note"),
            'callout': _("Only submitted official notices may be rejected."),
            'show_form': False
        }

    if form.submitted(request):
        self.reject(request, form.comment.data)
        request.message(_("Official notice rejected."), 'success')
        if self.user:
            request.app.send_email(
                subject=request.translate(
                    _(
                        "Official Notice Rejected ${id}",
                        mapping={'id': self.id}
                    )
                ),
                receivers=(self.user.username, ),
                reply_to=request.app.mail_sender,
                content=render_template(
                    'mail_notice_rejected.pt',
                    request,
                    {
                        'title': request.translate(_(
                            "Official Notice Rejected ${id}",
                            mapping={'id': self.id}
                        )),
                        'model': self,
                        'comment': form.comment.data,
                        'layout': MailLayout(self, request),
                        'url': request.link(self)
                    }
                )
            )
        return redirect(layout.dashboard_or_notices_link)

    return {
        'message': _(
            'Do you really want to reject "${item}"?',
            mapping={'item': self.title}
        ),
        'layout': layout,
        'form': form,
        'title': self.title,
        'subtitle': _("Reject Official Note"),
        'button_text': _("Reject Official Note"),
        'button_class': 'alert',
        'cancel': request.link(self)
    }
